/**	Author: Travis Scott (ID 1554515)
 * 	Date: 14/12/2012
 */

package uoa.tsco033.evolisa;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import android.os.Parcel;
import android.os.Parcelable;
import android.util.Log;

/**This class represents a drawing, and is composed of semi transparent polygons.
 * These polygons are split into three groups; background, mutable, and foreground. 
 * Mutable polygons can be mutated by the drawing, whereas the others are used for rendering
 * and fitness testing, but cannot be modified
 * @author tsco033
 */
public class DNADrawing implements Parcelable
{
	private List<DNAPolygon> backgroundPolygons;
	private List<DNAPolygon> mutablePolygons;
	private List<DNAPolygon> foregroundPolygons;
	private boolean isDirty;

	// ---------- Setup ----------//

	public DNADrawing()
	{
		backgroundPolygons = new ArrayList<DNAPolygon>();
		mutablePolygons = new ArrayList<DNAPolygon>();
		foregroundPolygons = new ArrayList<DNAPolygon>();

		for (int i = 0; i < Settings.ActivePolygonsMin; i++)
		{
			addPolygon();
		}

		setDirty();
	}

	public DNADrawing(List<DNAPolygon> backgroundPolygons, List<DNAPolygon> mutablePolygons, List<DNAPolygon> foregroundPolygons)
	{
		this.backgroundPolygons = new ArrayList<DNAPolygon>();
		this.mutablePolygons = new ArrayList<DNAPolygon>();
		this.foregroundPolygons = new ArrayList<DNAPolygon>();

		for (DNAPolygon polygon : backgroundPolygons)
		{
			this.backgroundPolygons.add(polygon.clone());
		}
		for (DNAPolygon polygon : mutablePolygons)
		{
			this.mutablePolygons.add(polygon.clone());
		}
		for (DNAPolygon polygon : foregroundPolygons)
		{
			this.foregroundPolygons.add(polygon.clone());
		}
	}

	public DNADrawing clone()
	{
		return new DNADrawing(backgroundPolygons, mutablePolygons, foregroundPolygons);
	}
	
/*
	public void initialise()
	{
		backgroundPolygons = new ArrayList<DNAPolygon>();
		mutablePolygons = new ArrayList<DNAPolygon>();
		foregroundPolygons = new ArrayList<DNAPolygon>();

		for (int i = 0; i < Settings.ActivePolygonsMin; i++)
		{
			addPolygon();
		}

		setDirty();
	}	
*/
	
	public void setPolygons(List<DNAPolygon> backgroundPolygons, List<DNAPolygon> mutablePolygons, List<DNAPolygon> foregroundPolygons)
	{
		this.backgroundPolygons = new ArrayList<DNAPolygon>();
		this.mutablePolygons = new ArrayList<DNAPolygon>();
		this.foregroundPolygons = new ArrayList<DNAPolygon>();

		for (DNAPolygon polygon : backgroundPolygons)
		{
			this.backgroundPolygons.add(polygon.clone());
		}
		for (DNAPolygon polygon : mutablePolygons)
		{
			this.mutablePolygons.add(polygon.clone());
		}
		for (DNAPolygon polygon : foregroundPolygons)
		{
			this.foregroundPolygons.add(polygon.clone());
		}
	}

	// ---------- Parcelable interface ----------//

	@Override
	public int describeContents()
	{
		return 0;
	}

	@Override
	public void writeToParcel(Parcel destination, int flags)
	{
		destination.writeTypedList(backgroundPolygons);
		destination.writeTypedList(mutablePolygons);
		destination.writeTypedList(foregroundPolygons);
		destination.writeBooleanArray(new boolean[] { isDirty });
	}

	public DNADrawing(Parcel source)
	{
		backgroundPolygons = source.createTypedArrayList(DNAPolygon.DNAPolygonCreator);
		mutablePolygons = source.createTypedArrayList(DNAPolygon.DNAPolygonCreator);
		foregroundPolygons = source.createTypedArrayList(DNAPolygon.DNAPolygonCreator);
		boolean[] temp = new boolean[1];
		source.readBooleanArray(temp);
		isDirty = temp[1];
	}

	public static final Parcelable.Creator<DNADrawing> DNADrawingCreator = new Parcelable.Creator<DNADrawing>()
	{
		public DNADrawing createFromParcel(Parcel source)
		{
			return new DNADrawing(source);
		}

		public DNADrawing[] newArray(int size)
		{
			return new DNADrawing[size];
		}
	};

	// ---------- Mutation ----------//

	/** This method mutates the drawing	**/
	public void mutate()
	{
		// Check if a polygon should be added
		if (Tools.willMutate(Settings.ActiveAddPolygonMutationRate))
		{
			addPolygon();
		}
		// Check if a polygon should be removed
		if (Tools.willMutate(Settings.ActiveRemovePolygonMutationRate))
		{
			removePolygon();
		}
		// Check if a polygon should be moved
		if (Tools.willMutate(Settings.ActiveMovePolygonMutationRate))
		{
			movePolygon();
		}

		// Mutate each polygon
		for (DNAPolygon polygon : mutablePolygons)
		{
			polygon.mutate(this);
		}
	}

	/** Adds a polygon to the drawing **/
	private void addPolygon()
	{
		// Check the polygon count is below the drawing maximum
		if (mutablePolygons.size() < Settings.PolygonsPerThreadMax)
		{
			// Create a new polygon
			DNAPolygon newPolygon = new DNAPolygon();
			newPolygon.initialise();

			// Generate a random index (position in list)
			int index = Tools.getRandomNumber(0, mutablePolygons.size());

			// Insert the polygon and flag the drawing as dirty
			mutablePolygons.add(index, newPolygon);
			setDirty();
		}
	}

	/** Removes a polygon from the drawing **/
	private void removePolygon()
	{
		// Check the polygon count is more than the drawing minimum
		if (mutablePolygons.size() > Settings.PolygonsPerThreadMin)
		{
			// Generate a random index
			int index = Tools.getRandomNumber(0, mutablePolygons.size() - 1);
			
			// Remove the polygon at this position and flag the drawing as dirty
			mutablePolygons.remove(index);
			setDirty();
		}
	}

	/** Moves a polygon in the list (changes z position) **/
	private void movePolygon()
	{
		// Return if there are no polygons in the list (shouldn't happen)
		if (mutablePolygons.size() < 1)
		{
			return;
		}

		// Generate a random index position
		int index = Tools.getRandomNumber(0, mutablePolygons.size() - 1);
		
		// Get the polygon at that position, and remove it from the list
		DNAPolygon polygon = mutablePolygons.get(index);
		mutablePolygons.remove(index);
		
		// Generate a random index position
		index = Tools.getRandomNumber(0, mutablePolygons.size() - 1);
		
		// Add the polygon at that point and flag the drawing as dirty
		mutablePolygons.add(index, polygon);
		setDirty();
	}

	// ---------- Methods ----------//

	/**Returns the total of number of points in the drawing 
	 * @return
	 */
	public int pointCount()
	{
		int pointCount = 0;
		for (DNAPolygon polygon : backgroundPolygons)
		{
			pointCount += polygon.pointCount();
		}
		for (DNAPolygon polygon : mutablePolygons)
		{
			pointCount += polygon.pointCount();
		}
		for (DNAPolygon polygon : foregroundPolygons)
		{
			pointCount += polygon.pointCount();
		}
		return pointCount;
	}

	/** Returns the total number of polygons in the drawing
	 * @return
	 */
	public int polygonCount()
	{
		int polygonCount = 0;
		for (DNAPolygon polygon : backgroundPolygons)
		{
			polygonCount++;
		}
		for (DNAPolygon polygon : mutablePolygons)
		{
			polygonCount++;
		}
		for (DNAPolygon polygon : foregroundPolygons)
		{
			polygonCount++;
		}
		return polygonCount;
	}

	/** Sets the flag to indicate the drawing has been modified **/
	public void setDirty()
	{
		this.isDirty = true;
	}

	/** Returns whether the drawing has been modified since it was last rendered and checked **/
	public boolean getDirty()
	{
		return isDirty;
	}

	// ---------- Accessors ----------//

	/** Returns a list of background polygons **/
	public List<DNAPolygon> getBackgroundPolygons()
	{
		return backgroundPolygons;
	}

	/** Returns a list of mutable polygons **/
	public List<DNAPolygon> getMutablePolygons()
	{
		return mutablePolygons;
	}

	/** Returns a list of foreground polygons **/
	public List<DNAPolygon> getForegroundPolygons()
	{
		return foregroundPolygons;
	}

	/** Returns a list of all polygons **/
	public List<DNAPolygon> getAllPolygons()
	{
		List<DNAPolygon> combinedPolygons = new ArrayList<DNAPolygon>();
		combinedPolygons.addAll(backgroundPolygons);
		combinedPolygons.addAll(mutablePolygons);
		combinedPolygons.addAll(foregroundPolygons);
		return combinedPolygons;
	}
	
}
