/**	Author: Travis Scott (ID 1554515)
 * 	Date: 14/12/2012
 */

package uoa.tsco033.evolisa;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileFilter;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import android.R.string;
import android.app.Activity;
import android.app.DialogFragment;
import android.app.Fragment;
import android.app.FragmentTransaction;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.media.MediaScannerConnection;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;

/** This class contains useful static functions 
 *  @author tsco033
 */
public class Tools
{
	private static final Random random = new Random();	// Random number generator, shared between all threads

	/** Returns a randomly generated integer, between a given minimum and maximum bound. 
	 * @param min
	 * @param max
	 * @return int - randomly generated between min and max inclusive
	 */
	public static int getRandomNumber(int min, int max)
	{
		// Check the maximum value is greater than 0
		if (max < 0) 
		{
			// If not set it to 0
			max = 0;
		}
		
		// Return a randomly generated integer
		return random.nextInt(max - min + 1) + min;
	}

	/** Determines if a mutation will occur for this generation for the given mutation rate.
	 * @param mutationRate
	 * @return boolean - Whether or not a mutation occured
	 */
	public static boolean willMutate(int mutationRate)
	{
		// Generates a random integer between 0 and the mutation rate, and returns true if 1 is generated
		// This mean a mutation will occur every (1/mutation rate) calls
		return (getRandomNumber(0, mutationRate) == 1);
	}
	
	//---------- File Operations ----------//
	
	private enum FileLocation
	{
		PICTURES, EXTERNAL;
	}
	
	private static File setupSaveFile(FileLocation location, String filename)
	{		
		File directory = null;
		
		switch (location)
		{
			case PICTURES:
				directory = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES), "/EvoLisa");
				break;
			case EXTERNAL:
				directory = new File(Environment.getExternalStorageDirectory(), "/EvoLisa");//"sdcard/external_sd");
				break;
		}
		
		File file = new File(directory, filename);
		
		// Check if the directory exists
		if (!directory.exists())
		{
			// If it doesn't exist try to create a new file
			if (!directory.mkdirs())
			{
				return null;
			}
		}
		
		// Check if the file exists
		if (!file.exists())
		{
			try
			{
				file.createNewFile();
			}
			catch (IOException exception)
			{
				exception.printStackTrace();
				return null;
			}
		}
		
		return file;
	}
	
	private static boolean writeStringToFile(File file, String text, boolean append)
	{			
		// Try and write the data to the file
		try
		{
			// Create a new BufferedWriter, setting append file flag
			BufferedWriter buf = new BufferedWriter(new FileWriter(file, append));
			// Perform the write
			buf.append(text);
			buf.newLine();
			buf.close();
			return true;
		}
		catch (IOException exception)
		{
			exception.printStackTrace();
			return false;
		}
	}
	
	private static boolean writeBitmapToFile(Context applicationContext, Bitmap bitmap, File file, Bitmap.CompressFormat format)
	{
		// Write the bitmap to file	
		try
		{
			FileOutputStream os = new FileOutputStream(file);
			
			switch (format)
			{
				case JPEG:					
					bitmap.compress(Bitmap.CompressFormat.JPEG, 100, os);
					os.flush();
					os.close();
					break;
				case PNG:					
					bitmap.compress(Bitmap.CompressFormat.PNG, 100, os);
					os.flush();
					os.close();
					break;
				default:
					os.flush();
					os.close();
					return false;
			}
			
			// Tell the OS to add the file to the media gallery
			MediaScannerConnection.scanFile(applicationContext, new String[] {file.getAbsolutePath()}, null, new MediaScannerConnection.OnScanCompletedListener()
			{				
				@Override
				public void onScanCompleted(String path, Uri uri)
				{
					Log.i("EvoLisa", "file " + path + " was scanned to gallery seccessfully: " + uri);				
				}
			});
			
			return true;
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return false;
		}
	}
	
	/** Appends the given text string to the log file on the external SD card 
	 * @param text
	 */
	public static void appendLog(String text)
	{		
		// Setup file
		File file = setupSaveFile(FileLocation.EXTERNAL, "Log.txt");
		
		if (file == null)
		{
			Log.i("EvoLisa", "Error creating to log file");
		}
		
		if (!writeStringToFile(file, text, true))
		{
			Log.i("EvoLisa", "Error writing to log file");
		}		
	}
	
	/** Clears the log file located on the external SD card. **/
	public static void clearLog()
	{
		// Setup file
		File file = setupSaveFile(FileLocation.EXTERNAL, "Log.txt");
		
		// Check if the file exists, and if it does delete it
		if (file.exists())
		{
			file.delete();
		}
	}
		
	public static String writeToSVGFile(DNADrawing drawing, String filename)
	{
		String xmlString;
		
		try
		{		
			// Create an empty XML Document
			DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
			Document doc = docBuilder.newDocument();
			
			// Create the root SVG element and add it to the document
			Element root = doc.createElementNS("http://www.w3.org/2000/svg", "svg");
			doc.appendChild(root);

			// Create an element for each polygon
			for (DNAPolygon polygon : drawing.getAllPolygons())
			{
				// Create a new element
				Element child = doc.createElement("polygon");
				
				// Add the coordinates
				StringBuilder coordinates = new StringBuilder();				
				for (DNAPoint point : polygon.getPoints())
				{
					coordinates.append(point.getX()).append(",").append(point.getY()).append(" ");
				}				
				child.setAttribute("points", coordinates.toString());
				
				// Add the fill colour
				int[] colourArray = polygon.getColor();
				StringBuilder colour = new StringBuilder();
				
				colour.append("fill:#");
				if (colourArray[1] < 16)
				{
					colour.append("0");
				}
				colour.append(Integer.toHexString(colourArray[1]));
				if (colourArray[2] < 16)
				{
					colour.append("0");
				}
				colour.append(Integer.toHexString(colourArray[2]));
				if (colourArray[3] < 16)
				{
					colour.append("0");
				}
				colour.append(Integer.toHexString(colourArray[3]));
				colour.append(";");
				
				colour.append("fill-opacity:").append(Double.toString(colourArray[0]/(double)255));
				
				child.setAttribute("style", colour.toString());
				
				// Add the element to the root
				root.appendChild(child);
			}

			// Output the XML

			// Set up a transformer
			TransformerFactory transfac = TransformerFactory.newInstance();
			Transformer trans = transfac.newTransformer();
			trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
			trans.setOutputProperty(OutputKeys.INDENT, "yes");

			// Create string from xml tree
			StringWriter sw = new StringWriter();
			StreamResult result = new StreamResult(sw);
			DOMSource source = new DOMSource(doc);
			trans.transform(source, result);
			xmlString = sw.toString();
		}
		catch (Exception exception)
		{
			exception.printStackTrace();
			return "Error generating SVG file";
		}
	
		// Setup file
		File file = setupSaveFile(FileLocation.PICTURES, filename + ".svg");

		if (file == null)
		{
			return "Error creating " + filename + ".svg";
		}
		
		if (writeStringToFile(file, xmlString, false))
		{
			return filename + ".svg saved successfully";
		}
		else
		{
			return "Error writing to " + filename + ".svg";
		}		
	}
	
	public static String writeToPNGFile(Context context, DNADrawing drawing, int height, int width, String filename)
	{
		// Create a bitmap object and canvas
		Bitmap bitmap = createBitmap(drawing, height, width);

		// Setup file
		File file = setupSaveFile(FileLocation.PICTURES, filename + ".png");

		if (file == null)
		{
			return "Error creating " + filename + ".png";
		}

		if (writeBitmapToFile(context, bitmap, file, Bitmap.CompressFormat.PNG))
		{
			return filename + ".png saved successfully";
		}
		else
		{
			return "Error writing to " + filename + ".png";
		}
	}
	
	public static String writeToJPEGFile(Context context, DNADrawing drawing, int height, int width, String filename)
	{
		// Create a bitmap object and canvas
		Bitmap bitmap = createBitmap(drawing, height, width);

		// Setup file
		File file = setupSaveFile(FileLocation.PICTURES, filename + ".jpeg");

		if (file == null)
		{
			return "Error creating " + filename + ".jpeg";
		}

		if (writeBitmapToFile(context, bitmap, file, Bitmap.CompressFormat.JPEG))
		{
			return filename + ".jpeg saved successfully";
		}
		else
		{
			return "Error writing to " + filename + ".jpeg";
		}
	}
	
	private static Bitmap createBitmap(DNADrawing drawing, int height, int width)
	{
		// Create a bitmap object and canvas
		Bitmap bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
		Canvas canvas = new Canvas(bitmap);
		
		// Render the bitmap
		Renderer.render(drawing, canvas, 1);
		
		return bitmap;
	}
}
